package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.base.service.IBaseWellheadCloudService;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadInput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadOutput;
import pps.cloud.space.service.ISpaceOptimizeLongCloudService;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.space.entity.SpaceInstitutionDetailEnt;
import pps.core.space.entity.SpaceInstitutionWellheadView;
import pps.core.space.enums.BusinessError;
import pps.core.space.mapper.SpaceInstitutionDetailMapper;
import pps.core.space.mapper.SpaceInstitutionWellheadViewMapper;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;

import java.math.BigDecimal;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 长期间开优化Cloud模块
 *
 * @author ZWT
 * @date 2023/09/13 14:14
 */
@XService
public class SpaceOptimizeLongCloudServiceImpl implements ISpaceOptimizeLongCloudService {

    /**
     * 长期间开优化--定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult optimizeLongJob(XContext context) {
        //取生效中的基础间开
        SpaceInstitutionDetailMapper detailMapper = context.getBean(SpaceInstitutionDetailMapper.class);
        List<SpaceInstitutionDetailEnt> detailEntList = detailMapper.selectList(new LambdaQueryWrapper<SpaceInstitutionDetailEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(SpaceInstitutionDetailEnt::getIsCurrentBasic, BusinessConstant.ZERO)
                .apply(" institution_end_date >= CURDATE() ")
        );
        if (CollUtil.isEmpty(detailEntList)) {
            //未发现可优化基础间开制度
            return XServiceResult.error(context, BusinessError.DidNotFindSpace);
        }
        Set<String> lineIds = new HashSet<>(32);
        Set<String> institutionIds = new HashSet<>(32);
        for (SpaceInstitutionDetailEnt spaceInstitutionDetailEnt : detailEntList) {
            lineIds.add(spaceInstitutionDetailEnt.getLineId());
            institutionIds.add(spaceInstitutionDetailEnt.getId());
        }
        //取生效中间开的所有井口
        SpaceInstitutionWellheadViewMapper wellheadMapper = context.getBean(SpaceInstitutionWellheadViewMapper.class);
        List<SpaceInstitutionWellheadView> spaceWellheadList = wellheadMapper.selectListByInstitutionIds(institutionIds);
        if (CollUtil.isEmpty(spaceWellheadList)) {
            //未发现可优化井口
            return XServiceResult.error(context, BusinessError.DidNotFindWellhead);
        }
        this.setServiceRatingForSpaceWellheadList(context,spaceWellheadList);



        for (SpaceInstitutionDetailEnt detail : detailEntList) {

        }
        return XServiceResult.OK;
    }

    /**
     * 为间开井口列表设置发电功率
     *
     * @param context           上下文
     * @param spaceWellheadList 空间井口清单
     */
    private void setServiceRatingForSpaceWellheadList(XContext context, List<SpaceInstitutionWellheadView> spaceWellheadList) {
        IBaseWellheadCloudService cloudService = context.getBean(IBaseWellheadCloudService.class);
        XListResult<DynamicQueryBaseWellheadOutput> result = cloudService.queryBaseWellheadListByParam(context,
                DynamicQueryBaseWellheadInput.builder()
                        .wellheadIds(spaceWellheadList.stream()
                                .map(SpaceInstitutionWellheadView::getWellheadId)
                                .collect(Collectors.toList())
                        )
                        .build());
        result.throwIfFail();
        List<DynamicQueryBaseWellheadOutput> wellheadOutputList = result.getResult();
        if (CollUtil.isEmpty(wellheadOutputList)) {
            return;
        }
        spaceWellheadList.forEach(s -> {
            s.setServiceRating(
                    wellheadOutputList.stream()
                            .filter(w -> StringUtils.equals(s.getWellheadId(), w.getId()))
                            .findAny()
                            .map(DynamicQueryBaseWellheadOutput::getServiceRating)
                            .orElse(BigDecimal.ZERO)
            );
        });
    }
}
